using MetadataExtractor;
using MetadataExtractor.Formats.Exif;
using MetadataExtractor.Formats.Exif.Makernotes;
using System.Globalization;

namespace View_by_Distance.Metadata.Models.Stateless;

internal abstract class Exif
{

    private static DateTime? GetDateTime(string? value)
    {
        DateTime? result;
        string dateTimeFormat = "yyyy:MM:dd HH:mm:ss";
        string alternateFormat = "ddd MMM dd HH:mm:ss yyyy";
        if (value is not null && DateTime.TryParse(value, out DateTime dateTime))
            result = dateTime;
        else if (value is not null && value.Length == dateTimeFormat.Length && DateTime.TryParseExact(value, dateTimeFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out dateTime))
            result = dateTime;
        else if (value is not null && value.Length == alternateFormat.Length && DateTime.TryParseExact(value, alternateFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out dateTime))
            result = dateTime;
        else
            result = null;
        return result;
    }

    private static Shared.Models.AviDirectory[] GetAviDirectories(IReadOnlyList<MetadataExtractor.Directory> directories)
    {
        List<Shared.Models.AviDirectory> results = [];
        Shared.Models.AviDirectory aviDirectory;
        IEnumerable<MetadataExtractor.Formats.Avi.AviDirectory> aviDirectories = directories.OfType<MetadataExtractor.Formats.Avi.AviDirectory>();
        foreach (MetadataExtractor.Formats.Avi.AviDirectory a in aviDirectories)
        {
            if (a.Tags.Count == 0)
                continue;
            DateTime? dateTimeOriginal;
            string? duration = a.GetDescription(MetadataExtractor.Formats.Avi.AviDirectory.TagDuration);
            string? height = a.GetDescription(MetadataExtractor.Formats.Avi.AviDirectory.TagHeight);
            string? width = a.GetDescription(MetadataExtractor.Formats.Avi.AviDirectory.TagWidth);
            if (a.TryGetDateTime(MetadataExtractor.Formats.Avi.AviDirectory.TagDateTimeOriginal, out DateTime checkDateTime))
                dateTimeOriginal = checkDateTime;
            else
                dateTimeOriginal = GetDateTime(a.GetString(MetadataExtractor.Formats.Avi.AviDirectory.TagDateTimeOriginal));
            if (dateTimeOriginal is null && duration is null && height is null && width is null)
                continue;
            aviDirectory = new(dateTimeOriginal, duration, height, width);
            results.Add(aviDirectory);
        }
        return results.ToArray();
    }

    private static Shared.Models.ExifDirectoryBase[] GetExifBaseDirectories(IReadOnlyList<MetadataExtractor.Directory> directories)
    {
        List<Shared.Models.ExifDirectoryBase> results = [];
        Shared.Models.ExifDirectoryBase exifDirectoryBase;
        IEnumerable<ExifDirectoryBase> exifBaseDirectories = directories.OfType<ExifDirectoryBase>();
        foreach (ExifDirectoryBase e in exifBaseDirectories)
        {
            if (e.Tags.Count == 0)
                continue;
            DateTime? dateTime;
            DateTime checkDateTime;
            DateTime? dateTimeOriginal;
            DateTime? dateTimeDigitized;
            string? aperture = e.GetDescription(ExifDirectoryBase.TagAperture);
            string? applicationNotes = e.GetDescription(ExifDirectoryBase.TagApplicationNotes);
            string? artist = e.GetDescription(ExifDirectoryBase.TagArtist);
            string? bitsPerSample = e.GetDescription(ExifDirectoryBase.TagBitsPerSample);
            string? bodySerialNumber = e.GetDescription(ExifDirectoryBase.TagBodySerialNumber);
            string? cameraOwnerName = e.GetDescription(ExifDirectoryBase.TagCameraOwnerName);
            string? compressedAverageBitsPerPixel = e.GetDescription(ExifDirectoryBase.TagCompressedAverageBitsPerPixel);
            string? compression = e.GetDescription(ExifDirectoryBase.TagCompression);
            string? copyright = e.GetDescription(ExifDirectoryBase.TagCopyright);
            string? documentName = e.GetDescription(ExifDirectoryBase.TagDocumentName);
            string? exifVersion = e.GetDescription(ExifDirectoryBase.TagExifVersion);
            string? exposureTime = e.GetDescription(ExifDirectoryBase.TagExposureTime);
            string? fileSource = e.GetDescription(ExifDirectoryBase.TagFileSource);
            string? imageDescription = e.GetDescription(ExifDirectoryBase.TagImageDescription);
            string? imageHeight = e.GetDescription(ExifDirectoryBase.TagImageHeight);
            string? imageNumber = e.GetDescription(ExifDirectoryBase.TagImageNumber);
            string? imageUniqueId = e.GetDescription(ExifDirectoryBase.TagImageUniqueId);
            string? imageWidth = e.GetDescription(ExifDirectoryBase.TagImageWidth);
            string? isoSpeed = e.GetDescription(ExifDirectoryBase.TagIsoSpeed);
            string? lensMake = e.GetDescription(ExifDirectoryBase.TagLensMake);
            string? lensModel = e.GetDescription(ExifDirectoryBase.TagLensModel);
            string? lensSerialNumber = e.GetDescription(ExifDirectoryBase.TagLensSerialNumber);
            string? make = e.GetDescription(ExifDirectoryBase.TagMake);
            string? makerNote = e.GetDescription(ExifDirectoryBase.TagMakernote);
            string? model = e.GetDescription(ExifDirectoryBase.TagModel);
            string? orientation = e.GetDescription(ExifDirectoryBase.TagOrientation);
            int? orientationValue = orientation is null ? null : e.GetInt32(ExifDirectoryBase.TagOrientation);
            string? rating = e.GetDescription(ExifDirectoryBase.TagRating);
            string? ratingPercent = e.GetDescription(ExifDirectoryBase.TagRatingPercent);
            string? securityClassification = e.GetDescription(ExifDirectoryBase.TagSecurityClassification);
            string? shutterSpeed = e.GetDescription(ExifDirectoryBase.TagShutterSpeed);
            string? software = e.GetDescription(ExifDirectoryBase.TagSoftware);
            string? timeZone = e.GetDescription(ExifDirectoryBase.TagTimeZone);
            string? timeZoneDigitized = e.GetDescription(ExifDirectoryBase.TagTimeZoneDigitized);
            string? timeZoneOriginal = e.GetDescription(ExifDirectoryBase.TagTimeZoneOriginal);
            string? userComment = e.GetDescription(ExifDirectoryBase.TagUserComment);
            string? winAuthor = e.GetDescription(ExifDirectoryBase.TagWinAuthor);
            string? winComment = e.GetDescription(ExifDirectoryBase.TagWinComment);
            string? winKeywords = e.GetDescription(ExifDirectoryBase.TagWinKeywords);
            string? winSubject = e.GetDescription(ExifDirectoryBase.TagWinSubject);
            string? winTitle = e.GetDescription(ExifDirectoryBase.TagWinTitle);
            string? xResolution = e.GetDescription(ExifDirectoryBase.TagXResolution);
            string? yResolution = e.GetDescription(ExifDirectoryBase.TagYResolution);
            if (e.TryGetDateTime(ExifDirectoryBase.TagDateTime, out checkDateTime))
                dateTime = checkDateTime;
            else
                dateTime = GetDateTime(e.GetString(ExifDirectoryBase.TagDateTime));
            if (e.TryGetDateTime(ExifDirectoryBase.TagDateTimeOriginal, out checkDateTime))
                dateTimeOriginal = checkDateTime;
            else
                dateTimeOriginal = GetDateTime(e.GetString(ExifDirectoryBase.TagDateTimeOriginal));
            if (e.TryGetDateTime(ExifDirectoryBase.TagDateTimeDigitized, out checkDateTime))
                dateTimeDigitized = checkDateTime;
            else
                dateTimeDigitized = GetDateTime(e.GetString(ExifDirectoryBase.TagDateTimeDigitized));
            if (userComment is not null && userComment.Length > 255)
                userComment = "...";
            if (aperture is null
                && applicationNotes is null
                && artist is null
                && bitsPerSample is null
                && bodySerialNumber is null
                && cameraOwnerName is null
                && compressedAverageBitsPerPixel is null
                && compression is null
                && copyright is null
                && dateTime is null
                && dateTimeDigitized is null
                && dateTimeOriginal is null
                && documentName is null
                && exifVersion is null
                && exposureTime is null
                && fileSource is null
                && imageDescription is null
                && imageHeight is null
                && imageNumber is null
                && imageUniqueId is null
                && imageWidth is null
                && isoSpeed is null
                && lensMake is null
                && lensModel is null
                && lensSerialNumber is null
                && make is null
                && makerNote is null
                && model is null
                && orientation is null
                && orientationValue is null
                && rating is null
                && ratingPercent is null
                && securityClassification is null
                && shutterSpeed is null
                && software is null
                && timeZone is null
                && timeZoneDigitized is null
                && timeZoneOriginal is null
                && userComment is null
                && winAuthor is null
                && winComment is null
                && winKeywords is null
                && winSubject is null
                && winTitle is null
                && xResolution is not null
                && yResolution is null)
                continue;
            exifDirectoryBase = new(aperture,
                                    applicationNotes,
                                    artist,
                                    bitsPerSample,
                                    bodySerialNumber,
                                    cameraOwnerName,
                                    compressedAverageBitsPerPixel,
                                    compression,
                                    copyright,
                                    dateTime,
                                    dateTimeDigitized,
                                    dateTimeOriginal,
                                    documentName,
                                    exifVersion,
                                    exposureTime,
                                    fileSource,
                                    imageDescription,
                                    imageHeight,
                                    imageNumber,
                                    imageUniqueId,
                                    imageWidth,
                                    isoSpeed,
                                    lensMake,
                                    lensModel,
                                    lensSerialNumber,
                                    make,
                                    makerNote,
                                    model,
                                    orientation,
                                    orientationValue,
                                    rating,
                                    ratingPercent,
                                    securityClassification,
                                    shutterSpeed,
                                    software,
                                    timeZone,
                                    timeZoneDigitized,
                                    timeZoneOriginal,
                                    userComment,
                                    winAuthor,
                                    winComment,
                                    winKeywords,
                                    winSubject,
                                    winTitle,
                                    xResolution,
                                    yResolution);
            results.Add(exifDirectoryBase);
        }
        return results.ToArray();
    }

    private static Shared.Models.FileMetadataDirectory[] GetFileMetadataDirectories(string file, IReadOnlyList<MetadataExtractor.Directory> directories)
    {
        List<Shared.Models.FileMetadataDirectory> results = [];
        Shared.Models.FileMetadataDirectory fileMetadataDirectory;
        IEnumerable<MetadataExtractor.Formats.FileSystem.FileMetadataDirectory> fileMetadataDirectories = directories.OfType<MetadataExtractor.Formats.FileSystem.FileMetadataDirectory>();
        foreach (MetadataExtractor.Formats.FileSystem.FileMetadataDirectory f in fileMetadataDirectories)
        {
            if (f.Tags.Count == 0)
                continue;
            DateTime? fileModifiedDate;
            string? fileName = f.GetDescription(MetadataExtractor.Formats.FileSystem.FileMetadataDirectory.TagFileName);
            string? fileSize = f.GetDescription(MetadataExtractor.Formats.FileSystem.FileMetadataDirectory.TagFileSize);
            if (f.TryGetDateTime(MetadataExtractor.Formats.FileSystem.FileMetadataDirectory.TagFileModifiedDate, out DateTime checkDateTime))
                fileModifiedDate = checkDateTime;
            else
                fileModifiedDate = GetDateTime(f.GetString(MetadataExtractor.Formats.FileSystem.FileMetadataDirectory.TagFileModifiedDate));
            if (fileName is null || !file.EndsWith(fileName))
                throw new NotSupportedException($"!{file}.EndsWith({fileName})");
            if (fileModifiedDate is null && fileName is null && fileSize is null)
                continue;
            fileMetadataDirectory = new(fileModifiedDate, fileName, fileSize);
            results.Add(fileMetadataDirectory);
        }
        return results.ToArray();
    }

    private static Shared.Models.GifHeaderDirectory[] GetGifHeaderDirectories(IReadOnlyList<MetadataExtractor.Directory> directories)
    {
        List<Shared.Models.GifHeaderDirectory> results = [];
        Shared.Models.GifHeaderDirectory gifHeaderDirectory;
        IEnumerable<MetadataExtractor.Formats.Gif.GifHeaderDirectory> gifHeaderDirectories = directories.OfType<MetadataExtractor.Formats.Gif.GifHeaderDirectory>();
        foreach (MetadataExtractor.Formats.Gif.GifHeaderDirectory g in gifHeaderDirectories)
        {
            if (g.Tags.Count == 0)
                continue;
            string? imageHeight = g.GetDescription(MetadataExtractor.Formats.Gif.GifHeaderDirectory.TagImageHeight);
            string? imageWidth = g.GetDescription(MetadataExtractor.Formats.Gif.GifHeaderDirectory.TagImageWidth);
            if (imageHeight is null && imageWidth is null)
                continue;
            gifHeaderDirectory = new(imageHeight, imageWidth);
            results.Add(gifHeaderDirectory);
        }
        return results.ToArray();
    }

    private static Shared.Models.GpsDirectory[] GetGpsDirectories(IReadOnlyList<MetadataExtractor.Directory> directories)
    {
        List<Shared.Models.GpsDirectory> results = [];
        Shared.Models.GpsDirectory gpsDirectory;
        IEnumerable<GpsDirectory> gpsDirectories = directories.OfType<GpsDirectory>();
        foreach (GpsDirectory g in gpsDirectories)
        {
            if (g.Tags.Count == 0)
                continue;
            DateTime? timeStamp;
            string? altitude = g.GetDescription(GpsDirectory.TagAltitude);
            string? latitude = g.GetDescription(GpsDirectory.TagLatitude);
            string? latitudeRef = g.GetDescription(GpsDirectory.TagLatitudeRef);
            string? longitude = g.GetDescription(GpsDirectory.TagLongitude);
            string? longitudeRef = g.GetDescription(GpsDirectory.TagLongitudeRef);
            if (g.TryGetDateTime(GpsDirectory.TagTimeStamp, out DateTime checkDateTime))
                timeStamp = checkDateTime;
            else
                timeStamp = GetDateTime(g.GetString(GpsDirectory.TagTimeStamp));
            if (altitude is null && latitude is null && latitudeRef is null && longitude is null && longitudeRef is null && timeStamp is null)
                continue;
            gpsDirectory = new(altitude,
                               latitude,
                               latitudeRef,
                               longitude,
                               longitudeRef,
                               timeStamp);
            results.Add(gpsDirectory);
        }
        return results.ToArray();
    }

    private static Shared.Models.JpegDirectory[] GetJpegDirectories(IReadOnlyList<MetadataExtractor.Directory> directories)
    {
        List<Shared.Models.JpegDirectory> results = [];
        Shared.Models.JpegDirectory jpegDirectory;
        IEnumerable<MetadataExtractor.Formats.Jpeg.JpegDirectory> jpegDirectories = directories.OfType<MetadataExtractor.Formats.Jpeg.JpegDirectory>();
        foreach (MetadataExtractor.Formats.Jpeg.JpegDirectory j in jpegDirectories)
        {
            if (j.Tags.Count == 0)
                continue;
            string? imageHeight = j.GetDescription(MetadataExtractor.Formats.Jpeg.JpegDirectory.TagImageHeight);
            string? imageWidth = j.GetDescription(MetadataExtractor.Formats.Jpeg.JpegDirectory.TagImageWidth);
            if (imageHeight is null && imageWidth is null)
                continue;
            jpegDirectory = new(imageHeight, imageWidth);
            results.Add(jpegDirectory);
        }
        return results.ToArray();
    }

    private static Shared.Models.MakernoteDirectory[] GetMakernoteDirectories(IReadOnlyList<MetadataExtractor.Directory> directories)
    {
        List<Shared.Models.MakernoteDirectory> results = [];
        Shared.Models.MakernoteDirectory makernoteDirectory;
        IEnumerable<AppleMakernoteDirectory> appleMakernoteDirectories = directories.OfType<AppleMakernoteDirectory>();
        foreach (AppleMakernoteDirectory appleMakernoteDirectory in appleMakernoteDirectories)
        {
            if (appleMakernoteDirectory.Tags.Count == 0)
                continue;
            string? cameraSerialNumber = null;
            string? firmwareVersion = null;
            string? qualityAndFileFormat = null;
            if (cameraSerialNumber is null && firmwareVersion is null && qualityAndFileFormat is null)
                continue;
            makernoteDirectory = new(cameraSerialNumber, firmwareVersion, qualityAndFileFormat);
            results.Add(makernoteDirectory);
        }
        IEnumerable<CanonMakernoteDirectory> canonMakernoteDirectories = directories.OfType<CanonMakernoteDirectory>();
        foreach (CanonMakernoteDirectory canonMakernoteDirectory in canonMakernoteDirectories)
        {
            if (canonMakernoteDirectory.Tags.Count == 0)
                continue;
            string? cameraSerialNumber = canonMakernoteDirectory.GetDescription(CanonMakernoteDirectory.TagModelId);
            string? firmwareVersion = canonMakernoteDirectory.GetDescription(CanonMakernoteDirectory.TagCanonFirmwareVersion);
            string? qualityAndFileFormat = canonMakernoteDirectory.GetDescription(CanonMakernoteDirectory.CameraSettings.TagQuality);
            if (cameraSerialNumber is null && firmwareVersion is null && qualityAndFileFormat is null)
                continue;
            makernoteDirectory = new(cameraSerialNumber, firmwareVersion, qualityAndFileFormat);
            results.Add(makernoteDirectory);
        }
        IEnumerable<NikonType2MakernoteDirectory> nikonType2MakernoteDirectories = directories.OfType<NikonType2MakernoteDirectory>();
        foreach (NikonType2MakernoteDirectory nikonType2MakernoteDirectory in nikonType2MakernoteDirectories)
        {
            if (nikonType2MakernoteDirectory.Tags.Count == 0)
                continue;
            string? cameraSerialNumber = nikonType2MakernoteDirectory.GetDescription(NikonType2MakernoteDirectory.TagCameraSerialNumber);
            string? firmwareVersion = nikonType2MakernoteDirectory.GetDescription(NikonType2MakernoteDirectory.TagFirmwareVersion);
            string? qualityAndFileFormat = nikonType2MakernoteDirectory.GetDescription(NikonType2MakernoteDirectory.TagQualityAndFileFormat);
            if (cameraSerialNumber is null && firmwareVersion is null && qualityAndFileFormat is null)
                continue;
            makernoteDirectory = new(cameraSerialNumber, firmwareVersion, qualityAndFileFormat);
            results.Add(makernoteDirectory);
        }
        IEnumerable<OlympusMakernoteDirectory> olympusMakernoteDirectories = directories.OfType<OlympusMakernoteDirectory>();
        foreach (OlympusMakernoteDirectory olympusMakernoteDirectory in olympusMakernoteDirectories)
        {
            if (olympusMakernoteDirectory.Tags.Count == 0)
                continue;
            string? cameraSerialNumber = olympusMakernoteDirectory.GetDescription(OlympusMakernoteDirectory.TagSerialNumber1);
            string? firmwareVersion = olympusMakernoteDirectory.GetDescription(OlympusMakernoteDirectory.TagBodyFirmwareVersion);
            string? qualityAndFileFormat = olympusMakernoteDirectory.GetDescription(OlympusMakernoteDirectory.TagJpegQuality);
            if (cameraSerialNumber is null && firmwareVersion is null && qualityAndFileFormat is null)
                continue;
            makernoteDirectory = new(cameraSerialNumber, firmwareVersion, qualityAndFileFormat);
            results.Add(makernoteDirectory);
        }
        IEnumerable<PanasonicMakernoteDirectory> panasonicMakernoteDirectories = directories.OfType<PanasonicMakernoteDirectory>();
        foreach (PanasonicMakernoteDirectory panasonicMakernoteDirectory in panasonicMakernoteDirectories)
        {
            if (panasonicMakernoteDirectory.Tags.Count == 0)
                continue;
            string? cameraSerialNumber = panasonicMakernoteDirectory.GetDescription(PanasonicMakernoteDirectory.TagInternalSerialNumber);
            string? firmwareVersion = panasonicMakernoteDirectory.GetDescription(PanasonicMakernoteDirectory.TagFirmwareVersion);
            string? qualityAndFileFormat = panasonicMakernoteDirectory.GetDescription(PanasonicMakernoteDirectory.TagQualityMode);
            if (cameraSerialNumber is null && firmwareVersion is null && qualityAndFileFormat is null)
                continue;
            makernoteDirectory = new(cameraSerialNumber, firmwareVersion, qualityAndFileFormat);
            results.Add(makernoteDirectory);
        }
        IEnumerable<SamsungType2MakernoteDirectory> samsungType2MakernoteDirectories = directories.OfType<SamsungType2MakernoteDirectory>();
        foreach (SamsungType2MakernoteDirectory samsungType2MakernoteDirectory in samsungType2MakernoteDirectories)
        {
            if (samsungType2MakernoteDirectory.Tags.Count == 0)
                continue;
            string? cameraSerialNumber = samsungType2MakernoteDirectory.GetDescription(SamsungType2MakernoteDirectory.TagSerialNumber);
            string? firmwareVersion = samsungType2MakernoteDirectory.GetDescription(SamsungType2MakernoteDirectory.TagFirmwareName);
            string? qualityAndFileFormat = null;
            if (cameraSerialNumber is null && firmwareVersion is null && qualityAndFileFormat is null)
                continue;
            makernoteDirectory = new(cameraSerialNumber, firmwareVersion, qualityAndFileFormat);
            results.Add(makernoteDirectory);
        }
        IEnumerable<SonyType6MakernoteDirectory> sonyType6MakernoteDirectories = directories.OfType<SonyType6MakernoteDirectory>();
        foreach (SonyType6MakernoteDirectory sonyType6MakernoteDirectory in sonyType6MakernoteDirectories)
        {
            if (sonyType6MakernoteDirectory.Tags.Count == 0)
                continue;
            string? cameraSerialNumber = null;
            string? firmwareVersion = null;
            string? qualityAndFileFormat = null;
            if (cameraSerialNumber is null && firmwareVersion is null && qualityAndFileFormat is null)
                continue;
            makernoteDirectory = new(cameraSerialNumber, firmwareVersion, qualityAndFileFormat);
            results.Add(makernoteDirectory);
        }
        return results.ToArray();
    }

    private static Shared.Models.PhotoshopDirectory[] GetPhotoshopDirectories(IReadOnlyList<MetadataExtractor.Directory> directories)
    {
        List<Shared.Models.PhotoshopDirectory> results = [];
        Shared.Models.PhotoshopDirectory photoshopDirectory;
        IEnumerable<MetadataExtractor.Formats.Photoshop.PhotoshopDirectory> photoshopDirectories = directories.OfType<MetadataExtractor.Formats.Photoshop.PhotoshopDirectory>();
        foreach (MetadataExtractor.Formats.Photoshop.PhotoshopDirectory p in photoshopDirectories)
        {
            if (p.Tags.Count == 0)
                continue;
            string? jpegQuality = p.GetDescription(MetadataExtractor.Formats.Photoshop.PhotoshopDirectory.TagJpegQuality);
            string? url = p.GetDescription(MetadataExtractor.Formats.Photoshop.PhotoshopDirectory.TagUrl);
            if (jpegQuality is null && url is null)
                continue;
            photoshopDirectory = new(jpegQuality, url);
            results.Add(photoshopDirectory);
        }
        return results.ToArray();
    }

    private static Shared.Models.PngDirectory[] GetPngDirectories(IReadOnlyList<MetadataExtractor.Directory> directories)
    {
        List<Shared.Models.PngDirectory> results = [];
        Shared.Models.PngDirectory pngDirectory;
        IEnumerable<MetadataExtractor.Formats.Png.PngDirectory> pngDirectories = directories.OfType<MetadataExtractor.Formats.Png.PngDirectory>();
        foreach (MetadataExtractor.Formats.Png.PngDirectory p in pngDirectories)
        {
            if (p.Tags.Count == 0)
                continue;
            string? imageHeight = p.GetDescription(MetadataExtractor.Formats.Png.PngDirectory.TagImageHeight);
            string? imageWidth = p.GetDescription(MetadataExtractor.Formats.Png.PngDirectory.TagImageWidth);
            string? textualData = p.GetDescription(MetadataExtractor.Formats.Png.PngDirectory.TagTextualData);
            if (imageHeight is null && imageWidth is null && textualData is null)
                continue;
            pngDirectory = new(imageHeight, imageWidth, textualData);
            results.Add(pngDirectory);
        }
        return results.ToArray();
    }

    private static Shared.Models.QuickTimeMovieHeaderDirectory[] GetQuickTimeMovieHeaderDirectoryDirectories(IReadOnlyList<MetadataExtractor.Directory> directories)
    {
        List<Shared.Models.QuickTimeMovieHeaderDirectory> results = [];
        Shared.Models.QuickTimeMovieHeaderDirectory quickTimeMovieHeaderDirectory;
        IEnumerable<MetadataExtractor.Formats.QuickTime.QuickTimeMovieHeaderDirectory> quickTimeMovieHeaderDirectories = directories.OfType<MetadataExtractor.Formats.QuickTime.QuickTimeMovieHeaderDirectory>();
        foreach (MetadataExtractor.Formats.QuickTime.QuickTimeMovieHeaderDirectory q in quickTimeMovieHeaderDirectories)
        {
            if (q.Tags.Count == 0)
                continue;
            DateTime? created;
            if (q.TryGetDateTime(MetadataExtractor.Formats.QuickTime.QuickTimeMovieHeaderDirectory.TagCreated, out DateTime checkDateTime))
                created = checkDateTime;
            else
                created = GetDateTime(q.GetString(MetadataExtractor.Formats.QuickTime.QuickTimeMovieHeaderDirectory.TagCreated));
            if (created is null)
                continue;
            quickTimeMovieHeaderDirectory = new(created);
            results.Add(quickTimeMovieHeaderDirectory);
        }
        return results.ToArray();
    }

    private static Shared.Models.QuickTimeTrackHeaderDirectory[] GetQuickTimeTrackHeaderDirectoryDirectories(IReadOnlyList<MetadataExtractor.Directory> directories)
    {
        List<Shared.Models.QuickTimeTrackHeaderDirectory> results = [];
        Shared.Models.QuickTimeTrackHeaderDirectory quickTimeTrackHeaderDirectory;
        IEnumerable<MetadataExtractor.Formats.QuickTime.QuickTimeTrackHeaderDirectory> quickTimeTrackHeaderDirectories = directories.OfType<MetadataExtractor.Formats.QuickTime.QuickTimeTrackHeaderDirectory>();
        foreach (MetadataExtractor.Formats.QuickTime.QuickTimeTrackHeaderDirectory q in quickTimeTrackHeaderDirectories)
        {
            if (q.Tags.Count == 0)
                continue;
            DateTime? created;
            if (q.TryGetDateTime(MetadataExtractor.Formats.QuickTime.QuickTimeTrackHeaderDirectory.TagCreated, out DateTime checkDateTime))
                created = checkDateTime;
            else
                created = GetDateTime(q.GetString(MetadataExtractor.Formats.QuickTime.QuickTimeTrackHeaderDirectory.TagCreated));
            if (created is null)
                continue;
            quickTimeTrackHeaderDirectory = new(created);
            results.Add(quickTimeTrackHeaderDirectory);
        }
        return results.ToArray();
    }

    private static Shared.Models.WebPDirectory[] GetWebPDirectories(IReadOnlyList<MetadataExtractor.Directory> directories)
    {
        List<Shared.Models.WebPDirectory> results = [];
        Shared.Models.WebPDirectory webPDirectory;
        IEnumerable<MetadataExtractor.Formats.WebP.WebPDirectory> webPDirectories = directories.OfType<MetadataExtractor.Formats.WebP.WebPDirectory>();
        foreach (MetadataExtractor.Formats.WebP.WebPDirectory w in webPDirectories)
        {
            if (w.Tags.Count == 0)
                continue;
            string? imageHeight = w.GetDescription(MetadataExtractor.Formats.WebP.WebPDirectory.TagImageHeight);
            string? imageWidth = w.GetDescription(MetadataExtractor.Formats.WebP.WebPDirectory.TagImageWidth);
            if (imageHeight is null && imageWidth is null)
                continue;
            webPDirectory = new(imageHeight, imageWidth);
            results.Add(webPDirectory);
        }
        return results.ToArray();
    }

    private static Shared.Models.ExifDirectory Covert(Shared.Models.FilePath filePath, Shared.Models.PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName? personKeyFormattedAndKeyTicksAndDisplayDirectoryName, IReadOnlyList<MetadataExtractor.Directory> directories, System.Drawing.Size? size)
    {
        Shared.Models.ExifDirectory result;
        Shared.Models.AviDirectory[] aviDirectories = GetAviDirectories(directories);
        Shared.Models.GpsDirectory[] gpsDirectories = GetGpsDirectories(directories);
        Shared.Models.PngDirectory[] pngDirectories = GetPngDirectories(directories);
        Shared.Models.JpegDirectory[] jpegDirectories = GetJpegDirectories(directories);
        Shared.Models.WebPDirectory[] webPDirectories = GetWebPDirectories(directories);
        Shared.Models.ExifDirectoryBase[] exifBaseDirectories = GetExifBaseDirectories(directories);
        Shared.Models.GifHeaderDirectory[] gifHeaderDirectories = GetGifHeaderDirectories(directories);
        Shared.Models.MakernoteDirectory[] MakernoteDirectories = GetMakernoteDirectories(directories);
        Shared.Models.PhotoshopDirectory[] photoshopDirectories = GetPhotoshopDirectories(directories);
        Shared.Models.FileMetadataDirectory[] fileMetadataDirectories = GetFileMetadataDirectories(filePath.FullName, directories);
        Shared.Models.QuickTimeMovieHeaderDirectory[] quickTimeMovieHeaderDirectories = GetQuickTimeMovieHeaderDirectoryDirectories(directories);
        Shared.Models.QuickTimeTrackHeaderDirectory[] quickTimeTrackHeaderDirectories = GetQuickTimeTrackHeaderDirectoryDirectories(directories);
        result = new(aviDirectories,
                     null,
                     exifBaseDirectories,
                     fileMetadataDirectories,
                     filePath,
                     gifHeaderDirectories,
                     gpsDirectories,
                     size?.Height,
                     jpegDirectories,
                     MakernoteDirectories,
                     personKeyFormattedAndKeyTicksAndDisplayDirectoryName,
                     photoshopDirectories,
                     pngDirectories,
                     quickTimeMovieHeaderDirectories,
                     quickTimeTrackHeaderDirectories,
                     webPDirectories,
                     size?.Width);
        return result;
    }

    internal static Shared.Models.ExifDirectory GetExifDirectory(Shared.Models.FilePath filePath, Stream stream, Shared.Models.PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName? personKeyFormattedAndKeyTicksAndDisplayDirectoryName)
    {
        Shared.Models.ExifDirectory result;
        IReadOnlyList<MetadataExtractor.Directory> directories = ImageMetadataReader.ReadMetadata(stream);
        System.Drawing.Size? size;
        if (!stream.CanSeek)
            size = null;
        else
        {
            try
            { size = Dimensions.GetDimensions(stream); }
            catch (Exception)
            { size = null; }
        }
        result = Covert(filePath, personKeyFormattedAndKeyTicksAndDisplayDirectoryName, directories, size);
        return result;
    }

}