using System.Text;

namespace View_by_Distance.Shared.Models.Stateless;

internal abstract class Id
{

    internal static int GetDeterministicHashCode(byte[] value)
    {
        int result;
        unchecked
        {
            int hash1 = (5381 << 16) + 5381;
            int hash2 = hash1;
            for (int i = 0; i < value.Length; i += 2)
            {
                hash1 = ((hash1 << 5) + hash1) ^ value[i];
                if (i == value.Length - 1)
                    break;
                hash2 = ((hash2 << 5) + hash2) ^ value[i + 1];
            }
            result = hash1 + (hash2 * 1566083941);
        }
        return result;
    }

    internal static byte GetHasIgnoreKeyword(FilePath filePath) =>
        (byte)(filePath.Id > -1 ? 8 : 2);

    internal static byte GetHasDateTimeOriginal(ResultSettings resultSettings, FilePath filePath) =>
        (byte)(!resultSettings.ValidVideoFormatExtensions.Contains(filePath.ExtensionLowered) ? filePath.Id > -1 ? 9 : 1 : filePath.Id > -1 ? 6 : 4);

    internal static byte GetMissingDateTimeOriginal(ResultSettings resultSettings, FilePath filePath) =>
        (byte)(!resultSettings.ValidVideoFormatExtensions.Contains(filePath.ExtensionLowered) ? filePath.Id > -1 ? 7 : 3 : filePath.Id > -1 ? 5 : 0);

    internal static int GetId(ResultSettings resultSettings, MetadataSettings metadataSettings, string intelligentId)
    {
        int result;
        StringBuilder results = new();
        if (metadataSettings.IntMinValueLength < (resultSettings.ResultAllInOneSubdirectoryLength + 2))
            throw new NotSupportedException();
        for (int i = intelligentId.Length - (resultSettings.ResultAllInOneSubdirectoryLength + 2); i > -1; i--)
            _ = results.Append(intelligentId[i]);
        _ = results.Append(intelligentId[^3]).Append(intelligentId[^2]);
        result = int.Parse(results.ToString());
        if (intelligentId[^1] is '0' or '1' or '2' or '3' or '4')
            result *= -1;
        else if (intelligentId[^1] is not '9' and not '8' and not '7' and not '6' and not '5')
            throw new NotSupportedException();
        return result;
    }

    internal static bool NameWithoutExtensionIsIdFormat(MetadataSettings metadataSettings, string fileNameFirstSegment)
    {
        bool result;
        if (fileNameFirstSegment.Length < 5 || fileNameFirstSegment.Length > metadataSettings.IntMinValueLength)
            result = false;
        else
        {
            bool skipOneAllAreNumbers = fileNameFirstSegment[1..].All(char.IsNumber);
            result = (skipOneAllAreNumbers && fileNameFirstSegment[0] == '-') || (skipOneAllAreNumbers && char.IsNumber(fileNameFirstSegment[0]));
        }
        return result;
    }

#pragma warning disable IDE0060
    internal static string GetIntelligentId(ResultSettings resultSettings, MetadataSettings metadataSettings, long id, string extensionLowered, bool? hasIgnoreKeyword, bool? hasDateTimeOriginal)
#pragma warning restore IDE0060
    {
        string result;
        StringBuilder stringBuilder = new();
        if (metadataSettings.IntMinValueLength < (resultSettings.ResultAllInOneSubdirectoryLength + 2))
            throw new NotSupportedException();
        int key;
        string value;
        List<char> resultAllInOneSubdirectoryChars = [];
        if (hasDateTimeOriginal is null)
        {
            key = 0;
            value = id.ToString().PadLeft(metadataSettings.IntMinValueLength, '0');
        }
        else if (id > -1)
        {
            if (!resultSettings.ValidVideoFormatExtensions.Contains(extensionLowered))
                key = hasIgnoreKeyword is not null && hasIgnoreKeyword.Value ? 8 : hasDateTimeOriginal.Value ? 9 : 7;
            else
                key = hasIgnoreKeyword is not null && hasIgnoreKeyword.Value ? throw new NotImplementedException() : hasDateTimeOriginal.Value ? 6 : 5;
            value = id.ToString().PadLeft(metadataSettings.IntMinValueLength, '0');
        }
        else
        {
            if (!resultSettings.ValidVideoFormatExtensions.Contains(extensionLowered))
                key = hasIgnoreKeyword is not null && hasIgnoreKeyword.Value ? 2 : hasDateTimeOriginal.Value ? 1 : 3;
            else
                key = hasIgnoreKeyword is not null && hasIgnoreKeyword.Value ? throw new NotImplementedException() : hasDateTimeOriginal.Value ? 4 : 0;
            value = id.ToString()[1..].PadLeft(metadataSettings.IntMinValueLength, '0');
        }
        for (int i = value.Length - resultSettings.ResultAllInOneSubdirectoryLength - 1; i > -1; i--)
            _ = stringBuilder.Append(value[i]);
        for (int i = value.Length - resultSettings.ResultAllInOneSubdirectoryLength; i < value.Length; i++)
            resultAllInOneSubdirectoryChars.Add(value[i]);
        result = $"{stringBuilder}{string.Join(string.Empty, resultAllInOneSubdirectoryChars)}{key}";
        return result;
    }

    internal static string GetPaddedId(ResultSettings resultSettings, MetadataSettings metadataSettings, int id, string extensionLowered, bool? hasIgnoreKeyword, bool? hasDateTimeOriginal, int? index)
    {
        string result;
        if (metadataSettings.Offset < 0)
            result = Guid.NewGuid().ToString();
        else
        {
            string intelligentId = GetIntelligentId(resultSettings, metadataSettings, id, extensionLowered, hasIgnoreKeyword, hasDateTimeOriginal);
            int check = GetId(resultSettings, metadataSettings, intelligentId);
            if (check != id)
                throw new NotSupportedException();
            result = index is null || metadataSettings.Offset == IId.DeterministicHashCode ? intelligentId : $"{metadataSettings.Offset + index}{intelligentId}";
        }
        return result;
    }

}